package Reimburse

import (
	"ERP_salary/logs"
	"git.baozhida.cn/ERP_libs/lib"
	"github.com/beego/beego/v2/adapter/orm"
	orm2 "github.com/beego/beego/v2/client/orm"

	//"github.com/beego/beego/v2/client/orm"
	"time"
)

// 报销明细
type ReimburseDetails struct {
	Id                 int       `orm:"column(ID);size(11);auto;pk"`
	T_reimburse_id     int       `orm:"index;size(64);null"`                                   // 报销id
	T_trip_start_time  string    `orm:"size(64);null"`                                         // 出差开始时间
	T_trip_end_time    string    `orm:"size(64);null"`                                         // 出差结束时间
	T_trip_origin      string    `orm:"size(128);null"`                                        // 出差起点
	T_trip_terminus    string    `orm:"size(128);null"`                                        // 出差起点
	T_fee_type         int       `orm:"size(11);null"`                                         // 费用类型
	T_fee_details      int       `orm:"size(11);null"`                                         // 费用明细
	T_money            float32   `orm:"digits(12);decimals(2)"`                                // 费用金额
	T_reasons          string    `orm:"size(1024);null"`                                       // 费用事由
	T_project          string    `orm:"size(1024);null"`                                       // 涉及项目
	T_img              string    `orm:"size(1024);null"`                                       // 图片链接
	T_State            int       `orm:"size(20);default(1)"`                                   // 0 删除 1 正常
	T_approval_money   float32   `orm:"digits(12);decimals(2)"`                                // 审批金额
	T_approval_opinion string    `orm:"size(1024);null"`                                       // 审批意见
	CreateTime         time.Time `orm:"column(create_time);type(timestamp);null;auto_now_add"` //auto_now_add 第一次保存时才设置时间
	UpdateTime         time.Time `orm:"column(update_time);type(timestamp);null;auto_now"`     //auto_now 每次 model 保存时都会对时间自动更新
}

func (t *ReimburseDetails) TableName() string {
	return "reimburse_details"
}
func init() {
	//注册模型
	orm.RegisterModel(new(ReimburseDetails))
}

type ReimburseDetails_R struct {
	Id                 int
	T_reimburse_id     int     // 报销id
	T_trip_start_time  string  // 出差开始时间
	T_trip_end_time    string  // 出差结束时间
	T_trip_origin      string  // 出差起点
	T_trip_terminus    string  // 出差起点
	T_fee_type         int     // 费用类型
	T_fee_type_name    string  // 费用类型名称
	T_fee_details      int     // 费用明细
	T_fee_details_name string  // 费用明细名称
	T_money            float32 // 费用金额
	T_reasons          string  // 费用事由
	T_project          string  // 涉及项目
	T_img              string  // 图片链接
	T_State            int     // 0 删除 1 正常
	T_approval_money   float32 // 审批总金额
	T_approval_opinion string  // 审批意见
}

// ---------------- 特殊方法 -------------------
func ReimburseDetailsToReimburseDetails_R(t ReimburseDetails) (r ReimburseDetails_R) {
	r.Id = t.Id
	r.T_reimburse_id = t.T_reimburse_id
	r.T_trip_start_time = t.T_trip_start_time
	r.T_trip_end_time = t.T_trip_end_time
	r.T_trip_origin = t.T_trip_origin
	r.T_trip_terminus = t.T_trip_terminus
	r.T_fee_type = t.T_fee_type
	r.T_fee_type_name = Read_FeeType_Get(t.T_fee_type)
	r.T_fee_details = t.T_fee_details
	r.T_fee_details_name = Read_FeeType_Get(t.T_fee_details)
	r.T_money = t.T_money
	r.T_reasons = t.T_reasons
	r.T_project = t.T_project
	r.T_img = t.T_img
	r.T_State = t.T_State
	r.T_approval_money = t.T_approval_money
	r.T_approval_opinion = t.T_approval_opinion
	return r
}

type ReimburseDetailsDaoImpl struct {
	orm orm.Ormer
}

func NewReimburseDetails(orm orm.Ormer) *ReimburseDetailsDaoImpl {
	return &ReimburseDetailsDaoImpl{orm: orm}
}

func (dao *ReimburseDetailsDaoImpl) Add_ReimburseDetails(r ReimburseDetails) (id int64, err error) {
	id, err = dao.orm.Insert(&r)
	if err != nil {
		logs.Error(lib.FuncName(), err)
	}
	return id, err
}

// 获取 ById
func (dao *ReimburseDetailsDaoImpl) Read_ReimburseDetailsList_ByT_reimburse_id(T_reimburse_id int) (r []ReimburseDetails, err error) {
	qs := dao.orm.QueryTable(new(ReimburseDetails))

	_, err = qs.Filter("T_reimburse_id", T_reimburse_id).Filter("T_State__gt", 0).All(&r)
	if err != nil {
		logs.Error(lib.FuncName(), err)
	}
	return
}

// 获取 ById
func (dao *ReimburseDetailsDaoImpl) Read_ReimburseDetailsById(id int) (r ReimburseDetails, err error) {
	qs := dao.orm.QueryTable(new(ReimburseDetails))

	err = qs.Filter("id", id).One(&r)
	if err != nil {
		logs.Error(lib.FuncName(), err)
	}
	return
}

// 修改
func (dao *ReimburseDetailsDaoImpl) Update_ReimburseDetails(m ReimburseDetails, cols ...string) error {
	_, err := dao.orm.Update(&m, cols...)
	if err != nil {
		logs.Error(lib.FuncName(), err)
		return err
	}
	return nil
}

// 删除
func (dao *ReimburseDetailsDaoImpl) Delete_ReimburseDetails(T_reimburse_id int) error {
	qs := dao.orm.QueryTable(new(ReimburseDetails))
	//_, err := qs.Filter("T_reimburse_id", T_reimburse_id).Update(map[string]interface{}{
	//	"T_State": 0,
	//})

	_, err := qs.Filter("T_reimburse_id", T_reimburse_id).Delete()
	if err != nil {
		logs.Error(lib.FuncName(), err)
	}
	return err
}

// 获取列表
func (dao *ReimburseDetailsDaoImpl) Read_ReimburseDetails_List(T_reimburse_id string) (r_ []ReimburseDetails) {
	var maps []ReimburseDetails
	// 也可以直接使用 Model 结构体作为表名
	qs := dao.orm.QueryTable(new(ReimburseDetails))
	// 过滤
	cond := orm.NewCondition()
	cond = cond.And("T_State__gt", 0)

	if len(T_reimburse_id) > 0 {
		cond = cond.And("T_reimburse_id", T_reimburse_id)
	}

	// 查询
	_, err := qs.SetCond((*orm2.Condition)(cond)).OrderBy("Id").All(&maps)

	if err != nil {
		logs.Error(lib.FuncName(), err)
		return
	}

	return maps
}
